#include <memory>
#include <exception>
#include <QLibrary>
#include <QDebug>
#include <QDir>
#include <QThread>

// Заголовочные файлы ДТО9
#include "ifptr.h"
#include "iinput.h"
#include "dto_errors.h"
#include "dto_const.h"

static const bool SCANNER_ON_FPRINT11 = false;
static const bool USE_SHOW_PROPERTIES = false;

#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
#  include <QTextCodec>

static class TrInit
{
public:
    TrInit()
    {
        QTextCodec::setCodecForTr(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForCStrings(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf-8"));
    }
} g_TrInit;
#endif

class Exception : public std::exception {
public:
    Exception(const QString &msg)
        : m_message(msg)
    {
    }

    virtual ~Exception() throw()
    {
    }

    virtual const char* what() const throw()
    {
        static QByteArray rawMessage = m_message.toUtf8();
        return rawMessage.data();
    }

private:
    QString m_message;
};

class Thread : public QThread
{
    Q_OBJECT
public:
    static void sleep(unsigned long secs)
    {
        QThread::sleep(secs);
    }
};

void checkError(TED::IDTOBase *iface)
{
    if(!iface)
        throw Exception("Invalid interface");

    int rc = EC_OK;
    iface->get_ResultCode(&rc);
    if(rc < 0)
    {
        QString resultDescription, badParamDescription;
        QVector<wchar_t> v(256);
        int size = iface->get_ResultDescription(&v[0], v.size());
        if (size <= 0)
            throw Exception("get_ResultDescription error");
        if (size > v.size())
        {
            v.clear();
            v.resize(size + 1);
            iface->get_ResultDescription(&v[0], v.size());
        }
        resultDescription = QString::fromWCharArray(&v[0]);
        if(rc == EC_INVALID_PARAM) {
            QVector<wchar_t> v(256);
            int size = iface->get_BadParamDescription(&v[0], v.size());
            if (size <= 0)
                throw Exception("get_BadParamDescription error");
            if (size > v.size())
            {
                v.clear();
                v.resize(size + 1);
                iface->get_ResultDescription(&v[0], v.size());
            }
            badParamDescription = QString::fromWCharArray(&v[0]);
        }

        if(badParamDescription.isEmpty())
            throw Exception(QObject::trUtf8("[%1] %2")
                            .arg(rc)
                            .arg(resultDescription));
        else
            throw Exception(QObject::trUtf8("[%1] %2 (%3)")
                            .arg(rc)
                            .arg(resultDescription)
                            .arg(badParamDescription));
    }
}

class Handler : public TED::AbstractEventHandler
{
public:
    virtual int on_data_event(void *p = 0)
    {
        TED::Input::IInput *iface = (TED::Input::IInput*)p;

        try
        {
            // Забираем данные
            QVector<wchar_t> v(64);
            int size = iface->get_ScanData(&v[0], v.size());
            if (size <= 0)
                throw Exception("get_ScanData error");
            if (size > v.size())
            {
                v.clear();
                v.resize(size + 1);
                iface->get_DeviceSettings(&v[0], v.size());
            }
            QString scanData = QString::fromWCharArray(&v[0], v.size());
            qDebug() << "Data = " << qPrintable(scanData);

            // Удаляем сообщение из буфера драйвера.
            // Если удаляете сообщение не сразу, то нужно сохранить значение свойства EventNumber (get_EventNumber()), а потом удалить по нему.
            iface->DeleteEvent();
        }
        catch(const std::exception &ex)
        {
            qDebug() << qPrintable(QString::fromUtf8(ex.what()));
        }
        return 0;
    }
};

int main()
{
    std::auto_ptr<Handler> eventHandler(new Handler);

#if defined(Q_OS_LINUX)
    QLibrary libFptr("../bins/linux-x64/libfptr.so");
    if(!libFptr.load())
    {
        libFptr.setFileName("../bins/linux-x86/libfptr.so");
        libFptr.load();
    }
#elif defined(Q_OS_WIN32)
    QLibrary libFptr("../bins/nt-x86-mingw/fptr.dll");
    libFptr.load();
#else
#   error "Unknown OS"
#endif

    if(!libFptr.isLoaded())
    {
        qDebug() << qPrintable(QObject::trUtf8("Не удалось загрузить библиотеку fptr - [%1]")
                               .arg(libFptr.errorString()));
        return 1;
    }


#if defined(Q_OS_LINUX)
    QLibrary libInput("../bins/linux-x64/libinput.so");
    if(!libInput.load())
    {
        libInput.setFileName("../bins/linux-x86/libinput.so");
        libInput.load();
    }
#elif defined(Q_OS_WIN32)
    QLibrary libInput("../bins/nt-x86-mingw/input.dll");
    libInput.load();
#else
#   error "Unknown OS"
#endif

    if(!libInput.isLoaded())
    {
        qDebug() << qPrintable(QObject::trUtf8("Не удалось загрузить библиотеку input - [%1]")
                               .arg(libInput.errorString()));
        return 1;
    }

    CreateFptrInterfacePtr createFptr = (CreateFptrInterfacePtr) libFptr.resolve("CreateFptrInterface");
    ReleaseFptrInterfacePtr releaseFptr = (ReleaseFptrInterfacePtr) libFptr.resolve("ReleaseFptrInterface");
    CreateInputInterfacePtr createInput = (CreateInputInterfacePtr) libInput.resolve("CreateInputInterface");
    ReleaseInputInterfacePtr releaseInput = (ReleaseInputInterfacePtr) libInput.resolve("ReleaseInputInterface");

    TED::Fptr::IFptr *ifptr = createFptr(DTO_IFPTR_VER1);
    if(!ifptr)
    {
        qDebug() << qPrintable("Cannot create Fptr object");
        return 1;
    }
    TED::Input::IInput *iinput = createInput(DTO_IINPUT_VER1);
    if(!iinput)
    {
        qDebug() << qPrintable("Cannot create Input object");
        return 1;
    }

    try
    {
        // Выставляем рабочий каталог. В нем дККМ и дУВ будут искать требуемые им библиотеки.
        QFileInfo finfo(libFptr.fileName());
        if(ifptr->put_DeviceSingleSetting(S_SEARCHDIR, finfo.absolutePath().toStdWString().c_str()) < 0)
            checkError(ifptr);
        if(ifptr->ApplySingleSettings() < 0)
            checkError(ifptr);
        if(iinput->put_DeviceSingleSetting(S_SEARCHDIR, finfo.absolutePath().toStdWString().c_str()) < 0)
            checkError(iinput);
        if(iinput->ApplySingleSettings() < 0)
            checkError(iinput);

        // Для настройки драйвера можно вызвать графическое окно настроек
        if(USE_SHOW_PROPERTIES)
        {
            if(ifptr->ShowProperties() < 0)
                checkError(ifptr);
            if(iinput->ShowProperties() < 0)
                checkError(iinput);
        }
        // Или настроить без него
        else
        {
            // COM17
            //if(ifptr->put_DeviceSingleSetting(S_PORT, 17) < 0)
                //checkError(ifptr);
            // USB. Можно указать положение на шине (USB$1-1.3, например), но тогда не нужно указывать Vid и Pid
            // Работа напрямую с USB - Linux Only!
            if(ifptr->put_DeviceSingleSetting(S_PORT, SV_PORT_USB) < 0)
                checkError(ifptr);
            if(ifptr->put_DeviceSingleSetting(S_VID, 0x2912) < 0)
                checkError(ifptr);
            if(ifptr->put_DeviceSingleSetting(S_PID, 0x0001) < 0)
                checkError(ifptr);
            if(ifptr->put_DeviceSingleSetting(S_MODEL, TED::Fptr::ModelFPrintPay01K) < 0)
                checkError(ifptr);
            if(ifptr->put_DeviceSingleSetting(S_PROTOCOL, TED::Fptr::ProtocolAtol30) < 0)
                checkError(ifptr);
            if(ifptr->put_DeviceSingleSetting(S_BAUDRATE, 115200) < 0)
                checkError(ifptr);
            if(ifptr->ApplySingleSettings() < 0)
                checkError(ifptr);

            // Для работы со стандратными УВ нужно настроить параметры связи с ними
            if(!SCANNER_ON_FPRINT11)
            {
#if defined(Q_OS_LINUX)
                // Для работы с клавиатурными устройствами под Linux используется event-интерфейс.
                // Укажите или eventX, или ссылку на него (например, из /dev/input/by-path).
                if(iinput->put_DeviceSingleSetting(S_PORT, L"/dev/input/event6") < 0)
                    checkError(iinput);
#elif defined(Q_OS_WIN32)
                // Для работы с клавиатурными устройствами под Windows нужно указать порт KBD
                if(iinput->put_DeviceSingleSetting(S_PORT, SV_PORT_KBD) < 0)
                    checkError(iinput);
#endif
                // Значение чувствительности. Чем меньше - тем быстрее будут приходить данные, но выше шанс часть потерять.
                if(iinput->put_DeviceSingleSetting(S_SENS, 50) < 0)
                    checkError(iinput);
            }
            // Для работы со сканером, подключенным к FPrint11, нужно установить специальный порт
            else
            {
                if(iinput->put_DeviceSingleSetting(S_PORT, SV_PORT_SOFT) < 0)
                    checkError(iinput);
            }
            // Параметры распознавания данных нужны в любом случае
            if(iinput->put_DeviceSingleSetting(S_MODEL, TED::Input::Scanner) < 0)
                checkError(iinput);
            if(iinput->put_DeviceSingleSetting(S_PFX, L"") < 0)
                checkError(iinput);
            if(iinput->put_DeviceSingleSetting(S_SFX, L"0D") < 0)
                checkError(iinput);
            if(iinput->ApplySingleSettings() < 0)
                checkError(iinput);

            // Для работы со сканером через FPrint11 необходимо связать дККМ и дУВ
            if(SCANNER_ON_FPRINT11)
            {
                void *hndl;
                ifptr->get_ScannerPortHandler(&hndl);
                iinput->put_ScannerPortHandler(hndl);
                iinput->get_ScannerEventHandler(&hndl);
                ifptr->put_ScannerEventHandler((TED::AbstractScannerEventHandler*)hndl);
            }

            // Устанавливаем callback, куда будут приходить данные
            if (iinput->put_EventHandler(eventHandler.get()) < 0)
                checkError(iinput);

            // Активируем дККМ
            if (ifptr->put_DeviceEnabled(1) < 0)
                checkError(ifptr);
            // Активируем дУВ
            if (iinput->put_DeviceEnabled(1) < 0)
                checkError(iinput);

            // Включаем выдачу данных
            if (iinput->put_DataEventEnabled(1) < 0)
                checkError(iinput);

            // Процедура сохранения актуальных настроек дУВ. Если пользуетесь ShowProperties(),
            // то ей можно вытащить актуальные настройки, чтобы потом передать из в драйвер через put_DeviceSettings(),
            // а не настраивать заново
            if(true)
            {
                QVector<wchar_t> v(256);
                int size = iinput->get_DeviceSettings(&v[0], v.size());
                if (size <= 0)
                    throw Exception("get_DeviceSettings error");
                if (size > v.size())
                {
                    v.clear();
                    v.resize(size + 1);
                    iinput->get_DeviceSettings(&v[0], v.size());
                }
                QString settings = QString::fromWCharArray(&v[0], v.size());
                // settings - строка с настройками
                qDebug() << qPrintable(settings);
            }

            // Для примера ждем 20 секунд, а потом выключаемся. Данные будет выводить Handler.
            Thread::sleep(20);
        }
    }
    catch(const std::exception &ex)
    {
        qDebug() << qPrintable(ex.what());
        releaseFptr(&ifptr);
        releaseInput(&iinput);
        return 1;
    }

    releaseFptr(&ifptr);
    releaseInput(&iinput);
    return 0;
}
