#include <ctime>
#include <exception>
#include <QLibrary>
#include <QDebug>
#include <QDir>

// Заголовочные файлы ДТО9
#include "iscale.h"
#include "dto_errors.h"
#include "dto_const.h"

#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
#  include <QTextCodec>

static class TrInit
{
public:
    TrInit()
    {
        QTextCodec::setCodecForTr(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForCStrings(QTextCodec::codecForName("utf-8"));
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf-8"));
    }
} g_TrInit;
#endif

class Exception : public std::exception {
public:
    Exception(const QString &msg)
        : m_message(msg)
    {
    }

    virtual ~Exception() throw()
    {
    }

    virtual const char* what() const throw()
    {
        static QByteArray rawMessage = m_message.toUtf8();
        return rawMessage.data();
    }

private:
    QString m_message;
};

void checkError(TED::IDTOBase *iface)
{
    if(!iface)
        throw Exception("Invalid interface");

    int rc = EC_OK;
    iface->get_ResultCode(&rc);
    if(rc < 0)
    {
        QString resultDescription, badParamDescription;
        QVector<wchar_t> v(256);
        int size = iface->get_ResultDescription(&v[0], v.size());
        if (size <= 0)
            throw Exception("get_ResultDescription error");
        if (size > v.size())
        {
            v.clear();
            v.resize(size + 1);
            iface->get_ResultDescription(&v[0], v.size());
        }
        resultDescription = QString::fromWCharArray(&v[0]);
        if(rc == EC_INVALID_PARAM) {
            QVector<wchar_t> v(256);
            int size = iface->get_BadParamDescription(&v[0], v.size());
            if (size <= 0)
                throw Exception("get_BadParamDescription error");
            if (size > v.size())
            {
                v.clear();
                v.resize(size + 1);
                iface->get_ResultDescription(&v[0], v.size());
            }
            badParamDescription = QString::fromWCharArray(&v[0]);
        }

        if(badParamDescription.isEmpty())
            throw Exception(QObject::trUtf8("[%1] %2")
                            .arg(rc)
                            .arg(resultDescription));
        else
            throw Exception(QObject::trUtf8("[%1] %2 (%3)")
                            .arg(rc)
                            .arg(resultDescription)
                            .arg(badParamDescription));
    }
}

int main()
{
#if defined(Q_OS_LINUX)
    QLibrary lib("../bins/linux-x64/libscale.so");
    if(!lib.load())
    {
        lib.setFileName("../bins/linux-x86/libscale.so");
        lib.load();
    }
#elif defined(Q_OS_WIN32)
    QLibrary lib("../bins/nt-x86-mingw/scale.dll");
    lib.load();
#else
#   error "Unknown OS"
#endif

    if(!lib.isLoaded())
    {
        qDebug() << qPrintable(QObject::trUtf8("Не удалось загрузить библиотеку scale - [%1]")
                               .arg(lib.errorString()));
        return 1;
    }

    CreateScaleInterfacePtr create = (CreateScaleInterfacePtr) lib.resolve("CreateScaleInterface");
    ReleaseScaleInterfacePtr release = (ReleaseScaleInterfacePtr) lib.resolve("ReleaseScaleInterface");

    TED::Scale::IScale *iscale = create(DTO_ISCALE_VER1);
    if(!iscale)
    {
        qDebug() << qPrintable("Cannot create Scale object");
        return 1;
    }

    try
    {
        // Выставляем рабочий каталог. В нем дЭВ будет искать требуемые ему библиотеки.
        QFileInfo finfo(lib.fileName());
        if(iscale->put_DeviceSingleSetting(S_SEARCHDIR, finfo.absolutePath().toStdWString().c_str()) < 0)
            checkError(iscale);
        if(iscale->ApplySingleSettings() < 0)
            checkError(iscale);

        // Для настройки драйвера можно вызвать графическое окно настроек
        if(false)
        {
            if(iscale->ShowProperties() < 0)
                checkError(iscale);
        }
        // Или настроить без него
        else
        {
            // COM 17
            if(iscale->put_DeviceSingleSetting(S_PORT, 17) < 0)
                checkError(iscale);
            if(iscale->put_DeviceSingleSetting(S_PORT, SV_PORT_USB) < 0)
                checkError(iscale);
            if(iscale->put_DeviceSingleSetting(S_MODEL, TED::Scale::ModelCASCommon) < 0)
                checkError(iscale);
            if(iscale->put_DeviceSingleSetting(S_BAUDRATE, 115200) < 0)
                checkError(iscale);
            if(iscale->ApplySingleSettings() < 0)
                checkError(iscale);
        }

        // Процедура сохранения актуальных настроек дЭВ. Если пользуетесь ShowProperties(),
        // то ей можно вытащить актуальные настройки, чтобы потом передать из в драйвер через put_DeviceSettings(),
        // а не настраивать заново
        if(true)
        {
            QVector<wchar_t> v(256);
            int size = iscale->get_DeviceSettings(&v[0], v.size());
            if (size <= 0)
                throw Exception("get_DeviceSettings error");
            if (size > v.size())
            {
                v.clear();
                v.resize(size + 1);
                iscale->get_DeviceSettings(&v[0], v.size());
            }
            QString settings = QString::fromWCharArray(&v[0], v.size());
            // settings - строка с настройками
            qDebug() << qPrintable(settings);
        }

        // Активируем дЭВ
        if (iscale->put_DeviceEnabled(1) < 0)
            checkError(iscale);

        // Считываем вес
        if(iscale->ReadWeight() < 0)
            checkError(iscale);
        double d;
        iscale->get_Weight(&d);
        qDebug() << "weight " << d;

        // Устанавливаем тару
        if(iscale->put_TareWeight(1.0) < 0)
            checkError(iscale);
        if(iscale->SetTareWeight() < 0)
            checkError(iscale);
    }
    catch(const std::exception &ex)
    {
        qDebug() << qPrintable(ex.what());
        release(&iscale);
        return 1;
    }

    release(&iscale);
    return 0;
}

